@@ -62,7 +62,8 @@ enum
   PROP_ME_METHOD,
   PROP_BUFSIZE,
   PROP_RTP_PAYLOAD_SIZE,
-  PROP_CFG_BASE
+  PROP_CFG_BASE,
+  PROP_COMPLIANCE,
 };
 
 #define GST_TYPE_ME_METHOD (gst_ffmpegvidenc_me_method_get_type())
@@ -92,14 +93,14 @@ static void gst_ffmpegvidenc_base_init (GstFFMpegVidEncClass * klass);
 static void gst_ffmpegvidenc_init (GstFFMpegVidEnc * ffmpegenc);
 static void gst_ffmpegvidenc_finalize (GObject * object);
 
+static gboolean gst_ffmpegvidenc_start (GstVideoEncoder * encoder);
 static gboolean gst_ffmpegvidenc_stop (GstVideoEncoder * encoder);
 static GstFlowReturn gst_ffmpegvidenc_finish (GstVideoEncoder * encoder);
 static gboolean gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
     GstVideoCodecState * state);
 static gboolean gst_ffmpegvidenc_propose_allocation (GstVideoEncoder * encoder,
     GstQuery * query);
-static gboolean gst_ffmpegvidenc_reset (GstVideoEncoder * encoder,
-    gboolean hard);
+static gboolean gst_ffmpegvidenc_flush (GstVideoEncoder * encoder);
 
 static GstCaps *gst_ffmpegvidenc_getcaps (GstVideoEncoder * encoder,
     GstCaps * filter);
@@ -206,16 +207,23 @@ gst_ffmpegvidenc_class_init (GstFFMpegVidEncClass * klass)
           "RTP Payload Size", "Target GOB length", 0, G_MAXINT, 0,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /* register additional properties, possibly dependent on the exact CODEC */
-  gst_ffmpeg_cfg_install_property (klass, PROP_CFG_BASE);
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_COMPLIANCE,
+      g_param_spec_enum ("compliance", "Compliance",
+          "Adherence of the encoder to the specifications",
+          GST_TYPE_FFMPEG_COMPLIANCE, FFMPEG_DEFAULT_COMPLIANCE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /* register additional properties, possibly dependent on the exact CODEC */
+  gst_ffmpeg_cfg_install_property (klass, PROP_COMPLIANCE);
+
+  venc_class->start = gst_ffmpegvidenc_start;
   venc_class->stop = gst_ffmpegvidenc_stop;
   venc_class->finish = gst_ffmpegvidenc_finish;
   venc_class->handle_frame = gst_ffmpegvidenc_handle_frame;
   venc_class->getcaps = gst_ffmpegvidenc_getcaps;
   venc_class->set_format = gst_ffmpegvidenc_set_format;
   venc_class->propose_allocation = gst_ffmpegvidenc_propose_allocation;
-  venc_class->reset = gst_ffmpegvidenc_reset;
+  venc_class->flush = gst_ffmpegvidenc_flush;
 
   gobject_class->finalize = gst_ffmpegvidenc_finalize;
 }
@@ -238,6 +246,7 @@ gst_ffmpegvidenc_init (GstFFMpegVidEnc * ffmpegenc)
   ffmpegenc->buffer_size = 512 * 1024;
   ffmpegenc->gop_size = DEFAULT_VIDEO_GOP_SIZE;
   ffmpegenc->rtp_payload_size = 0;
+  ffmpegenc->compliance = FFMPEG_DEFAULT_COMPLIANCE;
 
   ffmpegenc->lmin = 2;
   ffmpegenc->lmax = 31;
@@ -254,6 +263,7 @@ gst_ffmpegvidenc_finalize (GObject * object)
   gst_ffmpeg_cfg_finalize (ffmpegenc);
 
   /* clean up remaining allocated data */
+  gst_ffmpeg_avcodec_close (ffmpegenc->context);
   av_free (ffmpegenc->context);
   avcodec_free_frame (&ffmpegenc->picture);
 
@@ -292,10 +302,15 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   if (ffmpegenc->opened) {
     gst_ffmpeg_avcodec_close (ffmpegenc->context);
     ffmpegenc->opened = FALSE;
+    if (avcodec_get_context_defaults3 (ffmpegenc->context,
+            oclass->in_plugin) < 0) {
+      GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
+      return FALSE;
+    }
   }
 
   /* if we set it in _getcaps we should set it also in _link */
-  ffmpegenc->context->strict_std_compliance = -1;
+  ffmpegenc->context->strict_std_compliance = ffmpegenc->compliance;
 
   /* user defined properties */
   ffmpegenc->context->bit_rate = ffmpegenc->bitrate;
@@ -476,8 +491,10 @@ file_read_err:
 
 open_codec_fail:
   {
-    if (ffmpegenc->context->priv_data)
-      gst_ffmpeg_avcodec_close (ffmpegenc->context);
+    gst_ffmpeg_avcodec_close (ffmpegenc->context);
+    if (avcodec_get_context_defaults3 (ffmpegenc->context,
+            oclass->in_plugin) < 0)
+      GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
     if (ffmpegenc->context->stats_in)
       g_free (ffmpegenc->context->stats_in);
     GST_DEBUG_OBJECT (ffmpegenc, "avenc_%s: Failed to open libav codec",
@@ -488,6 +505,9 @@ open_codec_fail:
 pix_fmt_err:
   {
     gst_ffmpeg_avcodec_close (ffmpegenc->context);
+    if (avcodec_get_context_defaults3 (ffmpegenc->context,
+            oclass->in_plugin) < 0)
+      GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
     GST_DEBUG_OBJECT (ffmpegenc,
         "avenc_%s: AV wants different colourspace (%d given, %d wanted)",
         oclass->in_plugin->name, pix_fmt, ffmpegenc->context->pix_fmt);
@@ -504,6 +524,9 @@ bad_input_fmt:
 unsupported_codec:
   {
     gst_ffmpeg_avcodec_close (ffmpegenc->context);
+    if (avcodec_get_context_defaults3 (ffmpegenc->context,
+            oclass->in_plugin) < 0)
+      GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
     GST_DEBUG ("Unsupported codec - no caps found");
     return FALSE;
   }
@@ -645,10 +668,11 @@ alloc_fail:
   }
 }
 
-static void
+static GstFlowReturn
 gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
 {
   GstVideoCodecFrame *frame;
+  GstFlowReturn flow_ret = GST_FLOW_OK;
   GstBuffer *outbuf;
   gint ret_size;
 
@@ -656,7 +680,7 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
 
   /* no need to empty codec if there is none */
   if (!ffmpegenc->opened)
-    return;
+    goto done;
 
   while ((frame =
           gst_video_encoder_get_oldest_frame (GST_VIDEO_ENCODER (ffmpegenc)))) {
@@ -702,11 +726,16 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
       if (ffmpegenc->context->coded_frame->key_frame)
         GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
 
-      gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (ffmpegenc), frame);
+      flow_ret =
+          gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (ffmpegenc), frame);
     } else {
       gst_video_codec_frame_unref (frame);
     }
   }
+
+done:
+
+  return flow_ret;
 }
 
 
@@ -741,6 +770,9 @@ gst_ffmpegvidenc_set_property (GObject * object,
     case PROP_RTP_PAYLOAD_SIZE:
       ffmpegenc->rtp_payload_size = g_value_get_int (value);
       break;
+    case PROP_COMPLIANCE:
+      ffmpegenc->compliance = g_value_get_enum (value);
+      break;
     default:
       if (!gst_ffmpeg_cfg_set_property (object, value, pspec))
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -774,6 +806,9 @@ gst_ffmpegvidenc_get_property (GObject * object,
     case PROP_RTP_PAYLOAD_SIZE:
       g_value_set_int (value, ffmpegenc->rtp_payload_size);
       break;
+    case PROP_COMPLIANCE:
+      g_value_set_enum (value, ffmpegenc->compliance);
+      break;
     default:
       if (!gst_ffmpeg_cfg_get_property (object, value, pspec))
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -782,12 +817,28 @@ gst_ffmpegvidenc_get_property (GObject * object,
 }
 
 static gboolean
-gst_ffmpegvidenc_reset (GstVideoEncoder * encoder, gboolean hard)
+gst_ffmpegvidenc_flush (GstVideoEncoder * encoder)
 {
   GstFFMpegVidEnc *ffmpegenc = (GstFFMpegVidEnc *) encoder;
 
-  if (ffmpegenc->opened) {
+  if (ffmpegenc->opened)
     avcodec_flush_buffers (ffmpegenc->context);
+
+  return TRUE;
+}
+
+static gboolean
+gst_ffmpegvidenc_start (GstVideoEncoder * encoder)
+{
+  GstFFMpegVidEnc *ffmpegenc = (GstFFMpegVidEnc *) encoder;
+  GstFFMpegVidEncClass *oclass =
+      (GstFFMpegVidEncClass *) G_OBJECT_GET_CLASS (ffmpegenc);
+
+  /* close old session */
+  gst_ffmpeg_avcodec_close (ffmpegenc->context);
+  if (avcodec_get_context_defaults3 (ffmpegenc->context, oclass->in_plugin) < 0) {
+    GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
+    return FALSE;
   }
 
   return TRUE;
@@ -810,6 +861,10 @@ gst_ffmpegvidenc_stop (GstVideoEncoder * encoder)
     g_free (ffmpegenc->working_buf);
     ffmpegenc->working_buf = NULL;
   }
+  if (ffmpegenc->input_state) {
+    gst_video_codec_state_unref (ffmpegenc->input_state);
+    ffmpegenc->input_state = NULL;
+  }
 
   return TRUE;
 }
@@ -819,9 +874,7 @@ gst_ffmpegvidenc_finish (GstVideoEncoder * encoder)
 {
   GstFFMpegVidEnc *ffmpegenc = (GstFFMpegVidEnc *) encoder;
 
-  gst_ffmpegvidenc_flush_buffers (ffmpegenc, TRUE);
-
-  return GST_FLOW_OK;
+  return gst_ffmpegvidenc_flush_buffers (ffmpegenc, TRUE);
 }
 
 gboolean
